/************************************************************************/
/*  segy2ms -	convert a SEGY file to MiniSEED.			*/
/*									*/
/*	Douglas Neuhauser						*/
/*	Seismological Laboratory					*/
/*	University of California, Berkeley				*/
/*	doug@seismo.berkeley.edu					*/
/*									*/
/************************************************************************/

/*
 * Copyright (c) 1996-2000 The Regents of the University of California.
 * All Rights Reserved.
 * 
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for educational, research and non-profit purposes,
 * without fee, and without a written agreement is hereby granted,
 * provided that the above copyright notice, this paragraph and the
 * following three paragraphs appear in all copies.
 * 
 * Permission to incorporate this software into commercial products may
 * be obtained from the Office of Technology Licensing, 2150 Shattuck
 * Avenue, Suite 510, Berkeley, CA  94704.
 * 
 * IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY
 * FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES,
 * INCLUDING LOST PROFITS, ARISING OUT OF THE USE OF THIS SOFTWARE AND
 * ITS DOCUMENTATION, EVEN IF THE UNIVERSITY OF CALIFORNIA HAS BEEN
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * THE UNIVERSITY OF CALIFORNIA SPECIFICALLY DISCLAIMS ANY WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.  THE SOFTWARE
 * PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY OF
 * CALIFORNIA HAS NO OBLIGATIONS TO PROVIDE MAINTENANCE, SUPPORT,
 * UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
 */

#ifndef lint
static char sccsid[] = "$Id: segy2ms.c,v 1.3 2000/10/26 13:40:05 doug Exp $ ";
#endif

#include <stdio.h>
#include <string.h>
#include <malloc.h>

#include "version.h"

#ifndef	DEFAULT_NETWORK
#define	DEFAULT_NETWORK		"XX"
#endif

#ifndef	DEFAULT_OUTPUT_FORMAT
#define	DEFAULT_OUTPUT_FORMAT	STEIM1
#endif

#ifndef	DEFAULT_BLKSIZE
#define	DEFAULT_BLKSIZE		4096
#endif

#ifndef	DEFAULT_SEGY_FORMAT
#define	DEFAULT_SEGY_FORMAT	"BERKELEY"
#endif

#define	QUOTE(x)	#x
#define	STRING(x)	QUOTE(x)

char *syntax[] = {
"%s version " VERSION,
"%s  [-S station] [-C channel] [-N network] [-f b|p] [-m channel_map_file]",
"    [-b blksize] [-F format] [-o outfile] [-h] [input files...]",
"    where:",
"	-S station  Specify explicit station name.",
"	-C channel  Specify explicit channel name.",
"	-N network  Specify explicit network name.  Default network name is " 
		    DEFAULT_NETWORK ".",
"	-f b|p	    Specify input SEGY format: BERKELEY or PASSCAL.",
"		    Default SEGY format is " DEFAULT_SEGY_FORMAT ".",
"	-b blksize  Specifies output blksize.  Default blocksize=" STRING(DEFAULT_BLKSIZE) ".",
"	-F format   Specifies output format for MiniSEED data.",
"		    Default output format is " STRING(DEFAULT_OUTPUT_FORMAT) ".",
"	-o output_file",
"		    Specifies an explicit output filename for the data.",
"		    If no output file is specified, MiniSEED data will be",
"		    written to stdout.",
"	-m channel_map_file",
"		    Specifies a file that contains information for mapping",
"		    SEGY station and channel names to SEED station and",
"		    channel names.",
"	-h	    Help - prints syntax message.",
"	input_files...",
"		    A list of 0 or more input file containing SEGY data.",
"		    If no input files are specified, data is read from stdin.",
NULL };

#include "qlib2.h"

#define	EXTERN
#define	DEFINE

#include "segy.h"
#include "segy_reel.h"
#include "procs.h"
#include "externals.h"

/************************************************************************/
/*  main:   main program.						*/
/************************************************************************/
main (int argc, char **argv)
{
    int i, ival, npts;
    DATA_HDR *hdr;
    EXT_TIME et;
    float *input_data = NULL;	/* segy reader allocates buffer.	*/
    int *data = NULL;
    char *mseed = NULL;		/* allow qlib to allocate buffer.	*/
    char *p;
    int n, nleft;
    int status;
    int nblocks;
    SEGYBREELHEAD *bin_reel_header;
    int segy_fmt;
    int ntraces;
    char filename[2048];

    info = stdout;
    parse_cmdline(&argc,&argv);
    segy_fmt = strchr("Bb", segy_format[0]) ? 0 : 1;

    do {
	if (argc > 0) {
	    if ((input=fopen(*argv,"r")) == NULL) {
		fprintf (stderr, "Error opening file %s\n", *argv);
		exit(1);
	    }
	}
	else input = stdin;
	ntraces = read_segy_reel (input, &bin_reel_header, segy_fmt);
	if (ntraces <= 0) {
	    FATAL("Error reading SEGY reel header");
	}
    
	for (i=0; i<ntraces; i++) {
	    status = read_segy_trace(input, bin_reel_header, &hdr, &data, segy_fmt);
	    if (status != 1 || hdr == NULL | data == NULL)
		FATAL("error reading input file")

	    capnstr(hdr->network_id,DEFAULT_NETWORK,2);
	    if (strlen(station)>0) strcpy(hdr->station_id,station);
	    if (strlen(channel)>0) strcpy(hdr->channel_id,channel);
	    if (strlen(network)>0) strcpy(hdr->network_id,network);

	    /* Discard traces with no station name.			*/
	    if (strlen(hdr->station_id) == 0) continue;

	    hdr->blksize = blksize;
	    hdr->data_type = output_format;
	    npts = hdr->num_samples;
	    mseed = NULL;
	    status = ms_pack_data (hdr, NULL, npts, data, &nblocks, &mseed, 0, NULL);
	    if (status != npts) FATAL("Error packing data into MiniSEED.")

	    /* Write MiniSEED data to output file.			*/
	    et = int_to_ext(hdr->begtime);
	    /*
	    sprintf (filename, "%s.%s.%s.D.%04d.%03d.%02d%02d%02d",
		     hdr->station_id, hdr->network_id, hdr->channel_id,
		     et.year, et.doy, et.hour, et.minute, et.second);
	    if ((output = fopen (filename, "w")) == NULL)
		FATAL("Error opening output file\n");
	    */
	    n = nleft = nblocks;
	    p = (char *)mseed;
	    while (nleft > 0 && n > 0) {
		n = fwrite (p, blksize, nleft, output);
		p += n*blksize;
		nleft -= n;
	    }
	    if (nleft) {
		fprintf (info, "Error writing MiniSEED data - ");
		fprintf (info, "%d records unwritten\n", nleft);
		exit(1);
	    }
	    free_data_hdr (hdr);
	    free (data);
	    free (mseed);
	}
	free (bin_reel_header);
	fclose (input);
	--argc;
	++argv;
    } while (argc > 0);

    fclose (output);
    return(0);
}
